import xbmc
import xbmcvfs
import os
import urllib2
import datetime
import zlib

MAIN_URL = 'http://45.62.245.50/'

class FileFetcher(object):

    # Variables
    FETCH_ERROR = -1
    FETCH_NOT_NEEDED = 0
    FETCH_OK = 1
    TYPE_DEFAULT = 1
    TYPE_REMOTE = 2

    basePath = xbmc.translatePath(os.path.join('special://profile', 'addon_data', 'script.pragma'))
    filePath = ''
    fileUrl = ''
    addon = None
    fileType = TYPE_DEFAULT

    def __init__(self, fileName, addon):

        # Addon Variable
        self.addon = addon

        # If it's a Remote file
        if fileName.startswith(("http://", "https://", "ftp://", "sftp://", "ftps://", "smb://", "nfs://")):

            self.fileType = self.TYPE_REMOTE
            self.fileUrl = fileName
            self.filePath = os.path.join(self.basePath, fileName.split('/')[-1])

        else:

            self.fileType = self.TYPE_DEFAULT
            self.fileUrl = MAIN_URL + fileName
            self.filePath = os.path.join(self.basePath, fileName)

        # Make sure the folder exists otherwise make it
        if not os.path.exists(self.basePath):
            os.makedirs(self.basePath)

    def fetchFile(self):

        # Fallback Variables
        retVal = self.FETCH_NOT_NEEDED
        fetch = False

        # If the file doesn't exist - Fetch it
        if not os.path.exists(self.filePath):

            fetch = True

        else:

            # Interval Setting
            interval = int(self.addon.getSetting('xmltv.interval'))

            # If the interval is set to "Always"
            if interval == 0:

                fetch = True

            else:

                # Calculate the Modified Time of the file
                modTime = datetime.datetime.fromtimestamp(os.path.getmtime(self.filePath))
                td = datetime.datetime.now() - modTime

                # Calculate Interval Time - Android doesn't support .total_seconds()
                diff = (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10 ** 6

                # If the Interval Time has passed for any of the Intervals fetch the file
                if ((interval == 1 and diff >= 43200) or (interval == 2 and diff >= 86400) or (interval == 3 and diff >= 172800)):
                    fetch = True

        # If fetching is required
        if fetch:

            # Create a Temp File
            tmpFile = os.path.join(self.basePath, 'tmp')

            if self.fileType == self.TYPE_REMOTE:
                xbmc.log('[script.pragma] file is in remote location: %s' % self.fileUrl, xbmc.LOGDEBUG)
                if not xbmcvfs.copy(self.fileUrl, tmpFile):
                    xbmc.log('[script.pragma] Remote file couldn\'t be copied: %s' % self.fileUrl, xbmc.LOGERROR)
            else:
                f = open(tmpFile, 'wb')
                xbmc.log('[script.pragma] file is on the internet: %s' % self.fileUrl, xbmc.LOGDEBUG)
                tmpData = urllib2.urlopen(self.fileUrl)
                data = tmpData.read()
                if tmpData.info().get('content-encoding') == 'gzip':
                    data = zlib.decompress(data, zlib.MAX_WBITS + 16)
                f.write(data)
                f.close()
            if os.path.getsize(tmpFile) > 256:
                if os.path.exists(self.filePath):
                    os.remove(self.filePath)
                os.rename(tmpFile, self.filePath)
                retVal = self.FETCH_OK
                xbmc.log('[script.pragma] file %s was downloaded' % self.filePath, xbmc.LOGDEBUG)
            else:
                retVal = self.FETCH_ERROR
                
        return retVal